#!/usr/bin/env python
# -*- coding: utf8 -*-

from unipath import Path


class Info(object):
    __type__ = None
    __parents__ = None
    __contents__ = None

    def __init__(self, path):
        try:
            path = Path(path)

            self.__parents__ = path
            self.__contents__ = [d.name for d in path.listdir()]
        except:
            pass



#!/usr/bin/env python
# -*- coding: utf8 -*-

import inupypi


def test_search_path(tmpdir):
    dirs = ['dir1', 'dir2', 'dir3', 'dir4']

    for d in dirs:
        d = tmpdir.join(d)
        d.mkdir()

        assert inupypi.core.search_path(d.strpath.upper(),
                                        tmpdir.strpath) == d.strpath



#!/usr/bin/env python
# -*- coding: utf8 -*-

import base64
import flask.ext.testing
import inupypi
import pytest

from StringIO import StringIO


class Test_Repo(flask.ext.testing.TestCase):
    @pytest.fixture(autouse=True)
    def create_workspace(self, tmpdir):
        self.header = {'Authorization': 'Basic %s' %
                                        base64.b64encode('test:test')}
        passwd = 'test:$apr1$mNUntSci$AJ5OEID3.S4N1auTptrJC1'  # test:test
        self.temp = tmpdir
        self.repo = self.temp.join('repo').mkdir()
        self.htpasswd = self.temp.join('htpasswd')
        self.htpasswd.write(passwd)

    def create_app(self):
        app = inupypi.create_app()
        app.config['TESTING'] = True
        app.config['INUPYPI_REPO'] = str(self.repo)
        app.config['HTAUTH_HTPASSWD_PATH'] = str(self.htpasswd)
        app.config['HTAUTH_REALM'] = 'Testing Authentication'
        return app

    def test_missing_repo_path(self):
        self.repo.remove()
        resp = self.client.get('/')

        assert resp.status_code == 500
        assert '500: Internal Server Error' in resp.data
        assert 'Missing repository or package path!' in resp.data

    def test_empty_repo(self):
        repo = self.repo.join('repo')

        main_resp = self.client.get('/')
        repo_resp = self.client.get('/%s/' % repo.basename)

        assert main_resp.status_code == 200
        assert repo_resp.status_code == 404
        assert 'Path or File could not be found!' in repo_resp.data
        assert '%s</a>' % repo.basename not in main_resp.data

    def test_repo_exists(self):
        repo = self.repo.join('repo1').mkdir()
        self.create_app()

        main_resp = self.client.get('/')
        repo_resp = self.client.get('/%s/' % repo.basename)

        assert main_resp.status_code == 200
        assert repo_resp.status_code == 200
        assert '%s</a>' % repo.basename in main_resp.data

    def test_create_path(self):
        main_repo = self.repo.join('repo1')
        sub_repo = main_repo.join('sub_repo1')
        resp = self.client.get('/')

        assert resp.status_code == 200
        assert main_repo.basename not in resp.data

        resp = self.client.post('/admin/create_folder/',
                                data={'path': '/',
                                      'folder_name': main_repo.basename},
                                follow_redirects=True)

        assert resp.status_code == 401
        assert main_repo.check(dir=False)

        resp = self.client.post('/admin/create_folder/', headers=self.header,
                                data={'path': '/',
                                      'folder_name': main_repo.basename},
                                follow_redirects=True)

        assert resp.status_code == 200
        assert main_repo.check(dir=True)

        resp = self.client.post('/admin/create_folder/',
                                data={'folder_name': '%s' % sub_repo.basename,
                                      'path': main_repo.basename},
                                follow_redirects=True)

        assert resp.status_code == 401
        assert sub_repo.check(dir=False)

        resp = self.client.post('/admin/create_folder/', headers=self.header,
                                data={'folder_name': '%s' % sub_repo.basename,
                                      'path': main_repo.basename},
                                follow_redirects=True)

        assert resp.status_code == 200
        assert sub_repo.check(dir=True)

        resp = self.client.get('/%s/' % main_repo.basename)

        assert resp.status_code == 200
        assert sub_repo.basename in resp.data

    def test_create_path_if_exists(self):
        name = 'repo1'
        name_upper = name.upper()
        main_repo = self.repo.join(name)
        main_repo_upper = self.repo.join(name_upper)
        main_repo.mkdir()
        resp = self.client.get('/')

        assert resp.status_code == 200
        assert main_repo.basename in resp.data
        assert not main_repo_upper.check()

        resp = self.client.post('/admin/create_folder/',
                                data={'folder_name': '%s' %
                                      main_repo.basename.upper()})

        # checks authentication within method
        assert resp.status_code == 401
        assert not main_repo_upper.check()

        resp = self.client.post('/admin/create_folder/', headers=self.header,
                                data={'folder_name': '%s' %
                                      main_repo.basename.upper()})

        assert resp.status_code == 302
        assert not main_repo_upper.check()
        assert main_repo.basename in resp.headers.get('Location')

    def test_rename_path(self):
        repo = self.repo.join('repo1').mkdir()
        repo_rename = self.repo.join('repo2')

        resp = self.client.get('/')

        assert resp.status_code == 200
        assert repo.basename in resp.data

        resp = self.client.post('/admin/rename/', data={'current':
                                repo.basename, 'rename': repo_rename.basename},
                                follow_redirects=True)

        # checks authentication within method
        assert resp.status_code == 401
        assert repo.check(dir=True)

        resp = self.client.post('/admin/rename/', headers=self.header,
                                data={'current': repo.basename,
                                      'rename': repo_rename.basename},
                                follow_redirects=True)

        assert resp.status_code == 200
        assert not repo.check(dir=True)

        resp = self.client.get('/')

        assert resp.status_code == 200
        assert repo_rename.basename in resp.data
        assert repo.basename not in resp.data

    def test_remove_path(self):
        repo = self.repo.join('repo1').mkdir()
        resp = self.client.get('/')

        assert resp.status_code == 200
        assert repo.basename in resp.data

        resp = self.client.post('/admin/remove/', data={'item_path':
                                                        repo.basename},
                                follow_redirects=True)

        # checks authentication within method
        assert resp.status_code == 401
        assert repo.check(dir=True)

        resp = self.client.post('/admin/remove/', headers=self.header,
                                data={'item_path': repo.basename},
                                follow_redirects=True)

        assert resp.status_code == 200
        assert not repo.check(dir=True)

    def test_upload_file(self):
        repo = self.repo.join('repo').mkdir()
        upload_file = repo.join('uploaded_file.txt')

        assert not upload_file.check(file=True)

        resp = self.client.post('/admin/upload/',
                                data={'path': repo.basename,
                                      'file': (StringIO('File upload content'),
                                               upload_file.basename)},
                                follow_redirects=True)

        # checks authentication within method
        assert resp.status_code == 401
        assert upload_file.check(file=False)

        resp = self.client.post('/admin/upload/', headers=self.header,
                                data={'path': repo.basename,
                                      'file': (StringIO('File upload content'),
                                               upload_file.basename)},
                                follow_redirects=True)

        assert resp.status_code == 200
        assert upload_file.check(file=True)

        resp = self.client.get('/%s/' % repo.basename)

        assert upload_file.basename in resp.data



#!/usr/bin/env python
# -*- coding: utf8 -*-

from flask import Blueprint, abort, current_app, redirect, request, url_for
from flask.ext import htauth
from inupypi.core import sanitize_path, search_path
from unipath import Path
from werkzeug import secure_filename

admin = Blueprint('admin', __name__)


@admin.route('/admin/create_folder/', methods=['POST'])
@htauth.authenticated
def create_folder():
    base = Path(current_app.config.get('INUPYPI_REPO',
                Path('.', 'packages')))
    path = sanitize_path(request.form.get('path', ''))
    folder = sanitize_path(request.form.get('folder_name', ''))

    if Path(path, folder):
        temp_base = Path(base, path).absolute()
        create_path = Path(temp_base, folder)

        if create_path.exists():
            return redirect('%s' % Path(path, folder))
        search = search_path(folder, temp_base)

        if search:
            base = Path(base).absolute()
            return redirect('%s' % sanitize_path(search.replace(base, '')))

        try:
            create_path.mkdir(parents=True)
        except Exception, e:
            status = 'Failed to create %s' % e
            abort(500, status)
    if request.referrer:
        return redirect(request.referrer)
    return redirect(url_for('main.index'))


@admin.route('/admin/rename/', methods=['POST'])
@htauth.authenticated
def rename():
    base = Path(current_app.config.get('INUPYPI_REPO',
                Path('.', 'packages')))
    current = request.form.get('current')
    rename = request.form.get('rename')

    if current and rename:
        try:
            Path(base, current).rename(Path(base, rename))
        except Exception, e:
            status = 'Failed to rename %s' % e
            abort(500, status)
    if request.referrer:
        return redirect(request.referrer)
    return redirect(url_for('main.index'))


@admin.route('/admin/remove/', methods=['POST'])
@htauth.authenticated
def remove():
    base = Path(current_app.config.get('INUPYPI_REPO',
                Path('.', 'packages')))
    path = sanitize_path(request.form.get('item_path'))

    if path:
        try:
            Path(base, path).rmtree()
        except Exception, e:
            status = 'Failed to remove %s' % e
            abort(500, status)
    if request.referrer:
        return redirect(request.referrer)
    return redirect(url_for('main.index'))


@admin.route('/admin/upload/', methods=['POST'])
@htauth.authenticated
def upload():
    base = Path(current_app.config.get('INUPYPI_REPO',
                Path('.', 'packages')))
    path = request.form.get('path')
    upload_file = request.files.get('file')

    if upload_file and path:
        path = sanitize_path(path)

        try:
            filename = secure_filename(upload_file.filename)
            upload_file.save(Path(base, path, filename))
        except Exception, e:
            status = 'Failed to upload %s to %s: %s' % (upload_file.filename,
                                                        path, e)
            abort(500, status)
    if request.referrer:
        return redirect(request.referrer)
    return redirect(url_for('main.index'))



#!/usr/bin/env python
# -*- coding: utf8 -*-

import inupypi
import pytest


class Test_Dirs(object):
    @pytest.fixture(autouse=True)
    def create_workspace(self, tmpdir):
        self.temp = tmpdir
        self.core_dirs = inupypi.core.Dirs(self.temp.strpath)
        self.dirs = ['dir1', 'dir2', 'dir3', 'dir4']

    def test_class_instance(self):
        assert(isinstance(self.core_dirs, inupypi.core.Dirs))
        assert(self.core_dirs.__parents__ == self.temp.strpath)

    def test_dir_contents(self):
        assert(not self.core_dirs.__contents__)

        for dir_ in self.dirs:
            self.temp.join(dir_).mkdir()
            self.core_dirs = inupypi.core.Dirs(self.temp.strpath)

            assert(self.core_dirs.__parents__ == self.temp.strpath)
            assert(dir_ in self.core_dirs.__contents__)



#!/usr/bin/env python
# -*- coding: utf8 -*-

import flask.ext.testing
import inupypi
import pytest


class Test_Files(flask.ext.testing.TestCase):
    @pytest.fixture(autouse=True)
    def create_workspace(self, tmpdir):
        self.temp = tmpdir

    def create_app(self):
        app = inupypi.create_app()
        app.config['TESTING'] = True
        app.config['INUPYPI_REPO'] = str(self.temp)
        return app

    def test_missing_file(self):
        test_file = self.temp.join('file_a.txt')
        resp = self.client.get('/%s' % test_file.basename,
                               follow_redirects=True)

        assert resp.status_code == 404
        assert 'Path or File could not be found!' in resp.data

    def test_file_exists(self):
        repo = 'repo1'
        data = 'file content'

        test_repo = self.temp.join(repo).mkdir()
        test_file = test_repo.join('file_a.txt')
        test_file.write(data)
        resp = self.client.get('/%s/%s' % (repo.upper(),
                                           test_file.basename),
                               follow_redirects=True)
        content = resp.headers.get('Content-Disposition')

        assert resp.status_code == 200
        assert content == 'attachment; filename=%s' % test_file.basename
        assert data in resp.data



#!/usr/bin/env python
# -*- coding: utf8 -*-

import flask.ext.testing
import inupypi
import pytest


class Test_Basic(flask.ext.testing.TestCase):
    @pytest.fixture(autouse=True)
    def create_workspace(self, tmpdir):
        self.temp = tmpdir

    def create_app(self):
        app = inupypi.create_app()
        app.config['TESTING'] = True
        app.config['INUPYPI_REPO'] = str(self.temp)
        return app

    def test_index(self):
        resp = self.client.get('/')

        assert resp.status_code == 200
        assert '<table' in resp.data

    def test_about(self):
        resp = self.client.get('/about/')

        assert resp.status_code == 200
        assert 'About inupypi', resp.data



#!/usr/bin/env python
# -*- coding: utf8 -*-

from flask import Blueprint
from inupypi.views import render

error = Blueprint('errors', __name__)


@error.app_errorhandler(403)
def forbidden(err):
    return render('403.html', error=err), 403


@error.app_errorhandler(404)
def page_not_found(err):
    return render('404.html', error=err), 404


@error.app_errorhandler(500)
def internal_server_error(err):
    return render('500.html', error=err), 500



#!/usr/bin/env python
# -*- coding: utf8 -*-

from argparse import ArgumentParser
from flask import Flask
from flask.ext import htauth
from flask.ext.themes import setup_themes
from inupypi.views.admin import admin
from inupypi.views.error import error
from inupypi.views.main import main
from unipath import Path


def create_app(**config):
    app = Flask(__name__)
    app.config['HTAUTH_HTPASSWD_PATH'] = Path(__file__, '..',
                                              'htpasswd').absolute()
    app.config['HTAUTH_REALM'] = 'inupypi Authentication'
    app.config['THEME'] = app.config.get('THEME', 'inupypi')

    setup_themes(app)
    htauth.HTAuth(app)

    app.config.update(config)
    app.register_blueprint(admin)
    app.register_blueprint(error)
    app.register_blueprint(main)

    return app


def app():
    parser = ArgumentParser(description='inupypi Standalone Server')
    parser.add_argument('-d', '--DEBUG', action='store_true', default=False,
                        help='enable debug mode')
    parser.add_argument('-H', '--HOST', default='127.0.0.1',
                        help='Server Listen address')
    parser.add_argument('-p', '--PORT', default='8080',
                        help='Server Listen port')
    parser.add_argument('-t', '--HTAUTH_HTPASSWD_PATH',
                        default=Path(__file__, '..', 'htpasswd').absolute(),
                        help='htpasswd file for authentication.')
    parser.add_argument('INUPYPI_REPO', help='path to repository')
    args = vars(parser.parse_args())
    app = create_app(**args)
    app.run(host=args.get('HOST'), port=int(args.get('PORT')))


if __name__ == '__main__':
    app()



#!/usr/bin/env python
# -*- coding: utf8 -*-

from flask import current_app as app
from flask.ext.themes import render_theme_template


def render(template, **kwargs):
    return render_theme_template(app.config.get('THEME'), template, **kwargs)



#!/usr/bin/env python
# -*- coding: utf8 -*-

from flask import Blueprint, send_from_directory
from inupypi.core import content
from inupypi.views import render
from unipath import Path

main = Blueprint('main', __name__)


@main.route('/', defaults={'route': None})
@main.route('/<path:route>/')
def index(route):
    contents = content(route)

    if isinstance(contents, Path) and contents.isfile():
        return send_from_directory(contents.parent, str(contents.name),
                                   as_attachment=True)
    return render('index.html', contents=contents)


@main.route('/about/')
def about():
    return render('about.html')



#!/usr/bin/env python
# -*- coding: utf8 -*-

from flask import abort, current_app
from inupypi.core.exceptions import InuPyPI404Exception, InuPyPIMissingRepoPath
from pkg_resources import parse_version
from unipath import Path


class Dirs(object):
    def __init__(self, path):
        self.__parents__ = None
        self.__contents__ = None

        try:
            path = Path(path)

            self.__parents__ = path
            self.__contents__ = sorted([dir_.name for dir_ in path.listdir()
                                        if dir_.isdir() or dir_.isfile()],
                                       key=lambda v: parse_version(v))
        except:
            pass


def content(request=None):
    base = Path(current_app.config.get('INUPYPI_REPO',
                Path('.', 'packages')))

    if request:
        repo = Path(base, request)
    else:
        repo = base

    try:
        repo = repo.absolute()
        base = base.absolute()

        if not repo.exists():
            if base == repo:
                raise InuPyPIMissingRepoPath

            # sets the request to lowercase and compares it with
            # the existing items in the repository in lowercase
            repo = search_path(repo, base)

            if not repo:
                raise InuPyPI404Exception

        if repo.isdir():
            return Dirs(repo)
        if repo.isfile():
            return repo

    except InuPyPIMissingRepoPath:
        abort(500, 'Missing repository or package path!')
    except InuPyPI404Exception:
        abort(404, 'Path or File could not be found!')
    except:
        abort(500, 'Internal Server Error!')
    return repo


def sanitize_path(path):
    return path if not path.startswith('/') else path[1:]


def search_path(path, repo):
    try:
        repo = Path(repo).absolute()
        walk = list(repo.walk())

        for item in walk:
            if item.lower() == Path(repo, path).lower():
                return item
    except:
        pass
    return False



#!/usr/bin/env python
# -*- coding: utf8 -*-

import sys

from setuptools import find_packages, setup
from setuptools.command.test import test as TestCommand


class PyTest(TestCommand):
    def finalize_options(self):
        TestCommand.finalize_options(self)

        self.test_args = ['-sv', '--pyargs', self.test_suite]

    def run_tests(self):
        import pytest

        errno = pytest.main(self.test_args)
        sys.exit(errno)

setup(
    name='inupypi',
    version='0.3.1',
    packages=find_packages(),
    include_package_data=True,
    url='https://github.com/hallaj/inupypi',
    maintainer='Muhammad Hallaj Subery',
    maintainer_email='hallajs@gmail.com',
    license='BSD',
    description='A multiple repository PyPI server implementation',
    platforms='FreeBSD, Linux',
    test_suite='tests',
    tests_require=['Flask-Testing', 'pytest'],
    cmdclass={'test': PyTest},
    entry_points={'console_scripts': ['inupypi_server = inupypi:app']},
    install_requires=['Flask', 'Flask-Themes', 'Flask-HTAuth', 'argparse',
                      'unipath'])
